(* -*- mode: tuareg                                                        -*-
******************************************************************************
** $Id: chart.mli,v 1.6 2003/02/25 02:37:35 albro Exp $
******************************************************************************
**                                 chart.mli
**
**              Copyright (C) 2002, UCLA.  All Rights Reserved.
*****************************************************************************)

(** The chart for a bottom-up MCFG chart parser.  Chart items contain
  a category and 2n states for some n > 0 (the arity of the category
  the item represents).

  @author Daniel M. Albro
  @version $Revision: 1.6 $
  @since 11/21/02 - [da]: Initial Creation.
*)

type subitem = int array
    (** An item on the chart, as presented to the user (sans cat) *)

(****************************************************************************
 *            Chart Class: Init arg is the number of categories.            *
 ****************************************************************************)

(** Represents the chart for a chart parser.
  @param nc Number of categories in the grammar being used.
*)
class chart : 
  int ->
object
  method set_cat_info : int -> int -> int array -> unit
    (** Sets the tree depth and search order for the category [cat].
      
      @param cat A category (1 <= [cat] <= [nc]) 
      @param arity The arity (dimension) of the category 
      @param order The order to insert/lookup items of this category.
      For example, [[|0;2;1;3|]] would mean the trie for [cat] is
      arranged with the 0th state first, then the 2nd state, then the
      1st state, and finally the 3rd state (it is a map from the 0th
      state to a map from the 2nd state to a map from the 1st state to
      a set containing the 3rd state.)  
      @return nothing
    *)

  method show_cat_info : unit -> unit
    (** Displays the chart indexing information.
      @return nothing *)

  method chart_size : int
    (** @return the number of items in the chart *)

  method matches : int -> subitem -> subitem list
    (** 
      @return a list of all items of category [cat] that match the
              specification of [item] -- here, anything matches where 
              a field in [item] is -1, and all other values must match 
              exactly.
      @param cat Category of the item to match
      @param item States of the item to match, where a state can be -1 to
                  match any state.
    *)

  method contains : int -> subitem -> bool
    (** 
      @return whether the exact item <[cat],[item]> exists in the chart
      @param cat Category of item to find
      @param item States of item to find *)

  method add : int -> subitem -> bool
    (** Adds an item to the chart, if it isn't already there.
      @param cat Category of item to add
      @param item States of item to add
      @return whether the item was actually added or not (this would be false
              if the item was already present in the chart) *)
end (* chart *)

