(* -*- mode: tuareg                                                        -*-
******************************************************************************
** $Id: mcfggrammar.mli,v 1.4 2003/02/25 02:37:35 albro Exp $
******************************************************************************
**                              mcfggrammar.mli
**
**              Copyright (C) 2002, UCLA.  All Rights Reserved.
******************************************************************************)

(** Stream parser for MCFG grammars.
   The BNF for this is as follows:

  [rule ::= ID "-->" rhs]

  [rhs  ::= STRING | ID rhs2]

  [rhs2 ::= ID maps | maps]

  [maps ::= "[" map "]" maps | /*empty*/]

  [map  ::= INT "," INT map2]

  [map2 ::= ";" map | /*empty*/]

  @since 12/03/02 - [da]: Initial Creation.
  @version $Revision: 1.4 $
  @author Daniel M. Albro
*)   

val lexer : string -> Genlex.token Stream.t
(** String lexer.
  @param s String to parse
  @return a lexical analyzer for parsing the string [s]
*)

val file_lexer : string -> Genlex.token Stream.t
(** Lexer that reads from a named file
  @param filename Name of file to parse
  @return a lexical analyzer for parsing the file named by [filename]
*)

val parse_grammar : Grammar.grammar -> Genlex.token Stream.t -> unit
(** Parses an MCFG grammar 
  @param g Grammar object to fill out with the information being parsed,
           also provides information needed for parsing.
  @param s Stream (lexical analyzer) we're parsing
  @return nothing
*)

val parse_rule : Grammar.grammar -> Genlex.token Stream.t -> Grammar.rule
(** Parses an MCFG rule 
  @param g Grammar object to fill out with the information being parsed,
           also provides information needed for parsing.
  @param s Stream of tokens (lexical analyzer) from an MCFG.
  @return the rule we parsed.
*)


