(* -*- mode: tuareg                                                        -*-
******************************************************************************
** $Id: parsetree.mli,v 1.4 2003/02/25 02:37:35 albro Exp $
******************************************************************************
**                               parsetree.mli
**
**              Copyright (C) 2002, UCLA.  All Rights Reserved.
******************************************************************************)

(** 
  Functions used to recover the parse tree from a chart used to parse a
  sentence. 

  @author Daniel M. Albro
  @version $Revision: 1.4 $
  @since 12/20/02 - [da]: Initial Creation.
*)

(** Categories of a parse tree *)
type parse_cat = 
    ParseTerminal of string             
      (** A terminal (name) *)
  | ParseNonTerminal of string * int * int array 
      (** A non-terminal (name, id, states) *)

(** The branches of a parse tree *)
type parse_rhs = ParseRhs of parse_node list  
  (** list of child nodes *)

  (** An actual parse tree or subtree *)
and parse_node = ParseNode of parse_cat * parse_rhs list 
  (** node in a parse tree (category of mother, 
      list of alternatives for descendants) *)

val print_parse_tree : parse_node -> unit
(** Outputs a parse tree in human-readable format 
  @param tree Tree to output
  @return nothing
*)

val lookup_matches : int -> Chart.subitem -> Chart.chart -> Grammar.grammar 
  -> int -> Chart.subitem list
  (** Looks in the chart (sort of) for items that match <cat,item>,
    which can be a template specification (that is, some elements of
    item can be negative numbers, where -1 represents a wildcard and
    <-1 represents a variable (the variables are actually looked for
    directly in the chart).  Also looks up any empty category items
    which might match, since they aren't actually in the chart.

    @param cat Category of item to look for
    @param item Array of states (that is, covered sentence positions)
    of item to look for.
    @param ch Chart to look in
    @param g Grammar being used to parse a sentence
    @param sl Length of sentence being parsed
    @return list of items that match the specification <[cat],[item]>
  *)

val recover_parse_tree : Grammar.grammar -> int -> Chart.chart -> parse_node
  (** Recovers the parse tree for a successful parse of a sentence of
    length {i sl} using grammar {i g}, where the parse produced the
    chart {i ch}.  If the parse was not actually successful, raises {i
    Not_found}.  Otherwise, returns a parse node.

    @param g Relevant grammar
    @param sl Length of sentence that was parsed
    @param ch Chart that was used to parse the sentence
    @return parse tree representing the sentence parsed
    @raise Not_found if the parse had not actually been successful.
  *)
              


