
open Tk
open Tree
open Parsetree

let scroll_width = ref 1000
let scroll_height = ref 400

let scrolled_canvas t width height region =
  let c = Frame.create t in
  let canvas = Canvas.create c ~width ~height ~scrollregion:region in
  let xscroll = Scrollbar.create c ~orient:`Horizontal in
  let yscroll = Scrollbar.create c ~orient:`Vertical in
  let associate_sbs xsb ysb canv =
    Canvas.configure canv 
      ~xscrollcommand:(Scrollbar.set xsb)
      ~yscrollcommand:(Scrollbar.set ysb);
    Scrollbar.configure xsb ~command:(Canvas.xview canv);
    Scrollbar.configure ysb ~command:(Canvas.yview canv)
  in
    associate_sbs xscroll yscroll canvas;
    pack [xscroll] ~side:`Bottom ~fill:`X;
    pack [yscroll] ~side:`Right ~fill:`Y;
    pack [canvas] ~side:`Left ~fill:`Both ~expand:true;
    pack [c] ~side:`Top ~fill:`Both ~expand:true;
    canvas

let canvas_mark, canvas_drag =
  let canvas_x = ref 0 in
  let canvas_y = ref 0 in
    (fun canvas x y -> canvas_x := x; canvas_y := y),
    (fun canvas tag x y ->
       let dx = x - !canvas_x in
       let dy = y - !canvas_y in
       let iter tg = Canvas.move canvas tg ~x:dx ~y:dy in
         List.iter iter (Canvas.find canvas ~specs:[`Withtag(tag)]);
         canvas_x := x;
         canvas_y := y)
  
let add_tree tree canvas =
  let tree_tag = Tree.gensym "tag" in
    tree#set_location 50 50;
    tree#layout canvas (ref []) (ref []);
    tree#compute_node_positions canvas;
    tree#draw canvas;
    tree#associate_tag canvas tree_tag;
    tree#calculate_size();
    if tree#width > (!scroll_width - 100) then
      scroll_width := tree#width + 100;
    if tree#height > (!scroll_height - 100) then
      scroll_height := tree#height + 100;
    Canvas.configure canvas 
      ~scrollregion:(0, 0, !scroll_width, !scroll_height);
    let tag = tree#get_tag in
      Canvas.bind ~events:[`ButtonPressDetail(1)]
        ~fields:[`MouseX; `MouseY]
        ~action:(fun ev ->
                   canvas_mark canvas ev.ev_MouseX ev.ev_MouseY)
        canvas tag;
      Canvas.bind ~events:[`Modified([`Button1], `Motion)]
        ~fields:[`MouseX; `MouseY]
        ~action:(fun ev -> canvas_drag canvas (`Tag tree_tag) ev.ev_MouseX 
                   ev.ev_MouseY)
        canvas tag

let string_of_subitem states =
  let str = ref "[" in
    for i = 0 to (Array.length states) - 1 do
      if i > 0 then
        str := !str ^ ",";
      str := !str ^ (string_of_int states.(i))
    done;
    str := !str ^ "]";
    !str

let label_of_parse_cat = function
    ParseTerminal(str) -> str
  | ParseNonTerminal(str, id, states) ->
      str (* ^ (string_of_subitem states) *)

let rec tree_from_parse_tree pt =
  match pt with
      ParseNode(cat, []) ->
        let label = label_of_parse_cat cat in
        let label_node = new text_tree_node label in
          new tree (label_node :> tree_node)
    | ParseNode(cat, rhs) ->
        let label = label_of_parse_cat cat in
        let rhs1 = List.hd rhs in
        let label_node = new text_tree_node label in
        let tree = new tree (label_node :> tree_node) in 
          begin
            match rhs1 with
                ParseRhs(pts) ->
                  List.iter 
                  (fun sub_pt ->
                     let subtree = tree_from_parse_tree sub_pt in
                       tree#add_child subtree)
                  pts
          end;
          tree

let display_parse_tree pt =
  let t = openTk() in
  let canvas = scrolled_canvas t 300 200 (0, 0, 1000, 400) in
  let tree = tree_from_parse_tree pt in
    add_tree tree canvas;
    mainLoop()

