(* -*- mode: tuareg                                                        -*-
******************************************************************************
** $Id: tree.mli,v 1.1 2003/03/08 01:05:38 albro Exp $
******************************************************************************
**                                  tree.mli
**
**              Copyright (C) 2003, UCLA.  All Rights Reserved.
******************************************************************************
** Authors:
**      Daniel M. Albro
******************************************************************************
** Description:
**
******************************************************************************
** History:
**      03/03/03 - [da]: Initial Creation.
******************************************************************************
** Bugs:
******************************************************************************
** To Do:
*****************************************************************************)

val last : 'a list -> 'a

val gensym : string -> string

val init_gensym : unit -> unit

type tree_node_type = RectTreeNode | TextTreeNode

type side_point = { mutable x : int; mutable y : int }

class virtual tree_node :
  object
    val mutable m_dx : int
    val mutable m_dy : int
    val mutable m_height : int
    val mutable m_tags : Tk.tagOrId list
    val mutable m_type : tree_node_type
    val mutable m_width : int
    val mutable m_xchildshift : int
    method associate_tag : Widget.canvas Widget.widget -> string -> unit
    method collapse : Widget.canvas Widget.widget -> unit
    method drag : Widget.canvas Widget.widget -> int -> int -> unit
    method virtual draw : Widget.canvas Widget.widget -> int -> int -> unit
    method dx : int
    method dy : int
    method expand : Widget.canvas Widget.widget -> unit
    method get_node_size : Widget.canvas Widget.widget -> int * int
    method get_tag : Tk.tagOrId
    method height : int
    method is_collapsed : unit -> bool
    method is_drop_site : unit -> bool
    method is_simple_node : unit -> bool
    method node_type : tree_node_type
    method virtual recalc_node_size : Widget.canvas Widget.widget -> unit
    method set_delta : int -> int -> unit
    method set_xchildshift : int -> unit
    method width : int
    method xchildshift : int
  end

class rect_tree_node :
  int ->
  int ->
  object
    val mutable m_dx : int
    val mutable m_dy : int
    val mutable m_height : int
    val mutable m_tags : Tk.tagOrId list
    val mutable m_type : tree_node_type
    val mutable m_width : int
    val mutable m_xchildshift : int
    method associate_tag : Widget.canvas Widget.widget -> string -> unit
    method collapse : Widget.canvas Widget.widget -> unit
    method drag : Widget.canvas Widget.widget -> int -> int -> unit
    method draw : Widget.canvas Widget.widget -> int -> int -> unit
    method dx : int
    method dy : int
    method expand : Widget.canvas Widget.widget -> unit
    method get_node_size : Widget.canvas Widget.widget -> int * int
    method get_tag : Tk.tagOrId
    method height : int
    method is_collapsed : unit -> bool
    method is_drop_site : unit -> bool
    method is_simple_node : unit -> bool
    method node_type : tree_node_type
    method recalc_node_size : Widget.canvas Widget.widget -> unit
    method set_delta : int -> int -> unit
    method set_xchildshift : int -> unit
    method width : int
    method xchildshift : int
  end

class text_tree_node :
  string ->
  object
    val mutable m_dx : int
    val mutable m_dy : int
    val mutable m_height : int
    val mutable m_is_collapsed : bool
    val mutable m_tags : Tk.tagOrId list
    val mutable m_text : string
    val mutable m_text_height : int
    val mutable m_text_width : int
    val mutable m_type : tree_node_type
    val mutable m_width : int
    val mutable m_xchildshift : int
    method associate_tag : Widget.canvas Widget.widget -> string -> unit
    method collapse : Widget.canvas Widget.widget -> unit
    method drag : Widget.canvas Widget.widget -> int -> int -> unit
    method draw : Widget.canvas Widget.widget -> int -> int -> unit
    method dx : int
    method dy : int
    method expand : Widget.canvas Widget.widget -> unit
    method get_node_size : Widget.canvas Widget.widget -> int * int
    method get_tag : Tk.tagOrId
    method height : int
    method is_collapsed : unit -> bool
    method is_drop_site : unit -> bool
    method is_simple_node : unit -> bool
    method node_type : tree_node_type
    method recalc_node_size : Widget.canvas Widget.widget -> unit
    method set_delta : int -> int -> unit
    method set_text : Widget.canvas Widget.widget -> string -> unit
    method set_xchildshift : int -> unit
    method text : string
    method width : int
    method xchildshift : int
  end

class tree :
  tree_node ->
  object
    val mutable m_big_dx : int
    val mutable m_big_dy : int
    val mutable m_children : tree list
    val mutable m_collapsed_children : tree list
    val mutable m_height : int
    val mutable m_last_drawn_x : int option
    val mutable m_last_drawn_y : int option
    val mutable m_root : tree_node
    val mutable m_tags : Tk.tagOrId list
    val mutable m_width : int
    val mutable m_x : int
    val mutable m_x_max : int
    val mutable m_x_min : int
    val mutable m_y : int
    val mutable m_y_max : int
    method add_child : tree -> unit
    method associate_tag : Widget.canvas Widget.widget -> string -> unit
    method cnp_aux :
      Widget.canvas Widget.widget -> int -> int -> int -> int -> int -> unit
    method collapse : Widget.canvas Widget.widget -> unit
    method collapsed : bool
    method compute_node_positions : Widget.canvas Widget.widget -> unit
    method drag : Widget.canvas Widget.widget -> unit
    method draw : Widget.canvas Widget.widget -> unit
    method erase : Widget.canvas Widget.widget -> unit
    method expand : Widget.canvas Widget.widget -> unit
    method find_hitee :
      Widget.canvas Widget.widget ->
      int -> int -> tree -> int -> (tree * tree * int) option
    method get_tag : Tk.tagOrId
    method has_children : unit -> bool
    method height : int
    method max_x : int
    method max_y : int
    method min_x : int
    method min_y : int
    method calculate_size : unit -> unit
    method layout :
      Widget.canvas Widget.widget ->
      side_point list ref -> side_point list ref -> unit
    method remove_child : int -> unit
    method replace_child : int -> tree -> unit
    method root : tree_node
    method set_location : int -> int -> unit
    method translate_minmax : int -> unit
    method was_hit : Widget.canvas Widget.widget -> int -> int -> bool
    method width : int
    method x : int
    method y : int
  end
